#pragma once

#include <MiniSAX.h>

#include "Common.h"

/***********************************************
  Classes to parse adrenalin configuration data
 ***********************************************/

using	miniSAX::CMiniSAXException;
using	miniSAX::CContentHandler;
using	miniSAX::CAttribute;
using	miniSAX::CAttributeList;
using	miniSAX::CMiniSAXParser;

class CAdrenalinContentHandler : public CContentHandler {
public:
    CAdrenalinContentHandler(CMiniSAXParser* parser);
    virtual ~CAdrenalinContentHandler();

    virtual void startElement(const char* name, CAttributeList& attributes)
        throw (CMiniSAXException);
    virtual void endElement(const char* name)
        throw (CMiniSAXException);

    virtual void handleCharacters(const char* buffer, size_t len)
        throw (CMiniSAXException);

protected:
	CMiniSAXParser*	m_parser;

	enum HandleState {
		ROOT_STATE,

		ADRENALIN_STATE,

		SYSOP_STATE,

		SYSOP_NAME_STATE,
		SYSOP_ADDRESS_STATE,
		SYSOP_AKA_STATE,

		PATHS_STATE,

		PATHS_INBOUND_STATE,
		PATHS_BINKOUTBOUND_STATE,
		PATHS_FILEBOXES_STATE,
		PATHS_LONGFILEBOXES_STATE,
		PATHS_NETMAIL_STATE,
		PATHS_BADTICS_STATE,
		PATHS_NEWTICS_STATE,
		PATHS_PROCESSEDTICS_STATE,
		PATHS_AUTOCREATEDAREAS_STATE,

		ALIAS_STATE,

		LOG_STATE,

		LOG_COMMONFILE_STATE,
		LOG_REQUESTSFILE_STATE,
		LOG_SHOWMUSTGOON_STATE,
		LOG_LOGMSGNAMES_STATE,

		DESC_STATE,

		DESC_AUTODESCFILE_STATE,
		DESC_ALLOWMANYDESCTAGS_STATE,

		FILES_STATE,
		FILES_RENAMEPLAN_STATE,

		FILELIST_STATE,

		FILELIST_NAME_STATE,
		FILELIST_FORMAT_STATE,
		FILELIST_SORT_STATE,

		MAIL_STATE,

		MAIL_KILLPROCESSED_STATE,
		MAIL_REPLYATTRIBUTES_STATE,

		MAIL_REPLYATTRIBUTES_ATTR_STATE,

		MAIL_CREATIONLIMIT_STATE,

		MAIL_CREATIONLIMIT_KBYTES_STATE,
		MAIL_CREATIONLIMIT_LINES_STATE,

		MAIL_NONOTIFY_STATE,
		MAIL_NOTIFYONERRORS_STATE,
		MAIL_NOTIFYONWARNINGS_STATE,

		LNGFILE_STATE,
		FORWARDEXPIRATIONTIME_STATE,
		MAXTICAGE_STATE,

		WARNINGINSTEADOFERROR_STATE,
		HATCHORIGINADDRESS_STATE,
		ALLAKASEEN_STATE,
		HANDLEREPLACES_STATE,
		FREEDRIVESPACELIMIT_STATE,

		FILEFLAGS_STATE,

		FILEFLAGS_ONEVENT_STATE,
		FILEFLAGS_ONFILE_STATE
	};
	HandleState	m_state;

	struct EntryDescriptor {
		HandleState	m_state;
		LPCTSTR		m_name;
	};
	typedef std::vector<EntryDescriptor>	EntriesHolder;
	EntriesHolder	m_entriesHolder;

	// we process each limit using two phases: on start of element and on element data
	// so we need to remember drive path on start
	tstring	m_drivePath;

	// we receive execute event type from the attribute when reading file flags
	ExecuteEvent	m_executeEvent;

	// we receive file mask (and possibly area name) from the attribute when reading file flags
	tstring	m_fileMask;

	// ATTENTION: we compare pointers, not strings, so please submit only constants
	// this method throws exception if there is already an entry in m_entriesholder with the specified name
	// at the specified state
	void checkDup(LPCTSTR name, bool remember = true) throw(CMiniSAXException);
	// rememberes entry at the current state
	void rememberEntry(LPCTSTR name);
	// removes entry at the current state
	void removeEntry(LPCTSTR name);
	// removes all entries for current state
	void removeEntries();

	EntriesHolder::iterator	getEntry(LPCTSTR name);
	bool hasEntry(LPCTSTR name);

	bool parseYesNo(LPCTSTR flag) throw(CMiniSAXException);
};
