#include "pch.h"

#include "Adrenalin.h"
#include "Common.h"
#include "Misc.hpp"
#include "Temp.hpp"
#include "CfgParse.hpp"
#include "LinksManager.h"
#include "ListOp.h"
#include "Logger.h"
#include "AreasManager.h"

#include "Areas.h"


void CArea::getNameDesc(std::string& nameDesc) const {
	const int	LINE_LEN   = 78;
	const int	PREFIX_LEN = 40;

	nameDesc = "   ";
	nameDesc.append( name );
    if (description.size() > 0) {
		nameDesc.append( 1, ' ' );
		if (nameDesc.size() < PREFIX_LEN-1) {
			nameDesc.append( PREFIX_LEN-1 - nameDesc.size(), '.' );
		}
		if (nameDesc.size() < PREFIX_LEN) {
			nameDesc.append( 1, ' ' );
		}
		std::string	alignedDesc;
		int	firstPrefixLen = nameDesc.size() > PREFIX_LEN ? nameDesc.size() : PREFIX_LEN;
		CListOp::alignDescription( description.c_str(), alignedDesc, LINE_LEN, "\r", PREFIX_LEN, firstPrefixLen );
		nameDesc.append( alignedDesc );
	}
}


bool CArea::isValidName(const tstring& areaName) {
	LPCTSTR	szName = areaName.c_str();
	while (*szName != _T('\0')) {
		if (*szName == _T('\\') || *szName == _T('/') || *szName == _T(':') ||
			*szName == _T('*') || *szName == _T('?') ||
			*szName == _T('<') || *szName == _T('>') ||
			*szName == _T('"'))
			return (false);
		szName++;
	}
	return (true);
}


void CArea::clearDescFile(LPCTSTR szFileName) const {
	ifstream     in;
	ofstream     tempout;
	TCHAR        szDescFileName[MAX_PATH];
	TCHAR        szTempFileName[MAX_PATH];
	TCHAR        szWorkFileName[MAX_PATH];
	TCHAR        szBuf[LINE_SIZE];
	int          Len;
	int          i;
	bool         fSkipLongDesc;
    
	// set file names
	lstrcpy( szDescFileName, workingPath.c_str() );
	lstrcat( szDescFileName, CCommonData::getInstance()->getFileListName().c_str() );
	lstrcpy( szTempFileName, workingPath.c_str() );
	lstrcat( szTempFileName, _T("adrenalin.tmp") );
	Len = lstrlen( szFileName );

	// check name for spaces
	for (i = 0; szFileName[i] != _T('\0'); i++) {
		if (szFileName[i] == _T(' '))
			break;
	}
	if (szFileName[i] != _T('\0')) {
		lstrcpy( szWorkFileName, _T("\"") );
		lstrcat( szWorkFileName, szFileName );
		lstrcat( szWorkFileName, _T("\"") );
		Len += 2;
	} else {
		lstrcpy( szWorkFileName, szFileName );
	}
	// copy desc file into temp file, but skip lines with WorkFileName
	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		in.open( szDescFileName );
		if (!in) {
			break;
		}
		tempout.open( szTempFileName );
		if (!tempout) {
			in.close();
			break;
		}
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szBuf, LINE_SIZE );
			if (!in)
				continue;
			if (strnicmp( szBuf, szWorkFileName, Len) == 0)
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szBuf != ' ' && *szBuf != '>' && 
						*szBuf != '+') {
						fSkipLongDesc = false;
						tempout << szBuf << endl;
					}
				} else
					tempout << szBuf << endl;
			}
		}
		in.close();
		tempout.close();
		_tunlink( szDescFileName );
		_trename( szTempFileName, szDescFileName );
		break;
	} while (true);
}


CArea::CArea():
	group(NULL), path(), workingPath(), flags(),
	sendPriority(-1),
	receivePriority(-1),
	changed(false),
	fileFlags(),
	newFiles(),
	traffic(0),
	outboundDefined(false),
	outboundType(OT_BINKLEY),
	m_renameFilePlan(RFP_ABSENT)
{
}


CArea::~CArea() {
}


void CArea::setPath(const tstring& newPath) {
	path = newPath;

	// adjust working path

	if (path.size() > 0) {
		if (path[0] == TEXT('\\')) {
			workingPath = tstring(CAdrenalin::getStartupDrive()) + path;
		} else {
			if (path.size() < 2 || path[1] != TEXT(':')) {
				workingPath = tstring(CAdrenalin::getStartupPath()) + path;
			} else {
				workingPath = path;
			}
		}
	} else {
		workingPath = tstring(CAdrenalin::getStartupPath()) + path;
	}

	if (workingPath.size() > 0) {
		if (workingPath[workingPath.size()-1] != TEXT('\\')) {
			workingPath += TEXT("\\");
		}
	}
}


const tstring& CArea::getPath() const {
	return path;
}


void CArea::setFlag(int flag) {
	if (!hasFlag(flag)) {
		flags.push_back( flag );
	}
}


void CArea::resetFlag(int flag) {
	vector<int>::iterator	it = flags.begin();
	while (it != flags.end()) {
		if (flag == (*it)) {
			flags.erase( it );
			return;
		}
		it++;
	}
}


bool CArea::hasFlag(int flag) const {
	vector<int>::const_iterator	it = flags.begin();
	while (it != flags.end()) {
		if (flag == (*it))
			return true;
		it++;
	}
	return false;
}


bool CArea::canSend(ConstLinkPtr link) const {
	ConstTiePtr	tie = getTie(link);

	if (tie == NULL) {

		// not linked

		if (!link->canAccessGroup( group->getName() )) {
			// link cannot access area group
			return false;
		}
		if (link->getSendPriority() < sendPriority) {
			// link hasn't enough priority
			return (false);
		}
	} else {

		// linked

		if (tie->getRouteType() == ROUTE_NONE) {

			// no special routing

			if (link->getSendPriority() < sendPriority) {
				// link hasn't enough priority
				return (false);
			}
		} else {

			if ((tie->getRouteType() & ROUTE_SEND) == 0) {
				// routing doesn't allow link to send files
				return false;
			}

		}
	}

	return (true);
}


bool CArea::canReceive(ConstLinkPtr link) const {
	ConstTiePtr	tie = getTie(link);

	if (tie == NULL) {

		// not linked

		if (!link->canAccessGroup( group->getName() )) {
			// link cannot access area group
			return false;
		}
		if (link->getReceivePriority() < receivePriority) {
			// link hasn't enough priority
			return (false);
		}
	} else {

		// linked

		if (tie->getRouteType() == ROUTE_NONE) {

			// no special routing

			if (link->getReceivePriority() < receivePriority) {
				// link hasn't enough priority
				return (false);
			}
		} else {

			if ((tie->getRouteType() & ROUTE_RECEIVE) == 0) {
				// routing doesn't allow link to receive files
				return false;
			}
		}
	}

	return (true);
}


TiePtr CArea::getTie(ConstLinkPtr link) {
	int	count = ties.size();
	for (int i = 0; i < count; i++) {
		if (ties[i]->getLink() == link) {
			return ties[i];
		}
	}
	return NULL;
}


ConstTiePtr CArea::getTie(ConstLinkPtr link) const {
	int	count = ties.size();
	for (int i = 0; i < count; i++) {
		if (ties[i]->getLink() == link) {
			return ties[i];
		}
	}
	return NULL;
}


bool CArea::removeTie(ConstLinkPtr link) {
	for (VTies::iterator it = ties.begin(); it != ties.end(); ++it) {
		TiePtr	tie = (*it);
		if (tie->getLink() == link) {
			ties.erase( it );
			delete tie;
			return true;
		}
	}
	return false;
}


void CArea::clearDescFile(const VTString& vsFileNames) const {
	ifstream    in;
	ofstream    tempout;
	TCHAR       szDescFileName[MAX_PATH];
	TCHAR       szTempFileName[MAX_PATH];
	TCHAR       szBuf[LINE_SIZE];
	bool        fSkipLongDesc;
    
	// set file names
	lstrcpy( szDescFileName, workingPath.c_str() );
	lstrcat( szDescFileName, CCommonData::getInstance()->getFileListName().c_str() );
	lstrcpy( szTempFileName, workingPath.c_str() );
	lstrcat( szTempFileName, TEXT("adrenalin.tmp") );

	// enclose names with spaces in quotes

	VTString					vsWorkFileNames;
	VTString::const_iterator	itName;

	for (itName = vsFileNames.begin(); itName != vsFileNames.end(); itName++) {
		if (HasSpaces((*itName).c_str())) {
			vsWorkFileNames.push_back( tstring(_T("\"")) + (*itName) + TEXT("\"") );
		}
		else {
			vsWorkFileNames.push_back( *itName );
		}
	}

	// copy desc file into temp file, but skip lines with WorkFileName
	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		in.open( szDescFileName );
		if (!in) {
			break;
		}
		tempout.open( szTempFileName );
		if (!tempout) {
			in.close();
			break;
		}
		// prepare buffer for new line
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szBuf, LINE_SIZE );
			if (!in)
				continue;
			// find name
			for (itName = vsWorkFileNames.begin(); itName != vsWorkFileNames.end(); itName++) {
				if (_tcsnicmp( szBuf, (*itName).c_str(), (*itName).length() ) == 0)
					break;
			}
			if (itName != vsWorkFileNames.end())
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szBuf != ' ' && *szBuf != '>' && 
						*szBuf != '+') {
						fSkipLongDesc = false;
						tempout << szBuf << endl;
					}
				} else {
					tempout << szBuf << endl;
				}
			}
		}
		in.close();
		tempout.close();
		_tunlink( szDescFileName );
		_trename( szTempFileName, szDescFileName );
		break;
	} while (true);
}

void CArea::updateDescFile(LPCTSTR szFileName, LPCTSTR szFileDesc, filesize_t fileSize,
                           const VTString& listLDesc)
{
	ofstream	out;
	ifstream    in;
	ofstream    tempout;
	TCHAR       szDescFileName[MAX_PATH];
	TCHAR       szTempFileName[MAX_PATH];
	TCHAR       szWorkFileName[MAX_PATH];
	TCHAR       szBuf[LINE_SIZE];
	int         Len;
	int         i;
	bool        fSkipLongDesc;
    
	// set file names
	lstrcpy( szDescFileName, workingPath.c_str() );
	lstrcat( szDescFileName, CCommonData::getInstance()->getFileListName().c_str() );
	lstrcpy( szTempFileName, workingPath.c_str() );
	lstrcat( szTempFileName, TEXT("adrenalin.tmp") );
	Len = lstrlen( szFileName );

	// check name for spaces
	for (i = 0; szFileName[i] != _T('\0'); i++) {
		if (szFileName[i] == _T(' '))
			break;
	}
	if (szFileName[i] != _T('\0')) {
		lstrcpy( szWorkFileName, _T("\"") );
		lstrcat( szWorkFileName, szFileName );
		lstrcat( szWorkFileName, _T("\"") );
		Len += 2;
	}
	else
		lstrcpy( szWorkFileName, szFileName );

	// copy desc file into temp file, but skip lines with WorkFileName
	BOOL fSuccessOverwrite = TRUE;

	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		in.open( szDescFileName );
		if (!in) {
			break;
		}
		tempout.open( szTempFileName );
		if (!tempout) {
			in.close();
			break;
		}
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szBuf, LINE_SIZE );
			if (!in) {
				if (!in.eof())
					break;
				continue;
			}

			// skip empty lines
			if (*szBuf == '\0')
				continue;

			if ((strnicmp( szBuf, szWorkFileName, Len) == 0) && ((szBuf[Len] == ' ') || (szBuf[Len] == '\0')))
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szBuf != ' ' && *szBuf != '>' && *szBuf != '+') {
						fSkipLongDesc = false;
						tempout << szBuf << endl;
						if (!tempout) {
							fSuccessOverwrite = false;
							break;
						}
					}
				} else {
					tempout << szBuf << endl;
					if (!tempout) {
						fSuccessOverwrite = false;
						break;
					}
				}
			}
		}
		in.close();
		tempout.close();
		if (fSuccessOverwrite) {
			_tunlink( szDescFileName );
			_trename( szTempFileName, szDescFileName );
		} else {
			_tunlink( szTempFileName );
			LoggerPtr	logger;
			logger << LOG_ERROR << TEXT("failed to update ") << szDescFileName << TEXT("\n");
		}
		break;
	} while (true);
    
	out.open( szDescFileName, ios::app );
	if (!out) {
		out.open( szDescFileName );
	}
	if (!out) {
		return;
	}

	strcpy( szBuf, CCommonData::getInstance()->getFileListFormat().apply( szFileName, 
		szFileDesc, listLDesc, fileSize).c_str() );

	out << szBuf << endl;
	out.close();

	// if we need to sort filelist suspend current area to be sorted on exit

	if (CCommonData::getInstance()->doSortFileList()) {
		CAreasManager::getInstance()->rememberArea( this );
	}
}

struct FileListEntry {
	VTString*	lines;
	size_t		index;
	size_t		length;
};

static int compareFileListEntries( const void *arg1, const void *arg2 ) {
	const FileListEntry*	entry1 = (const FileListEntry*)arg1;
	const FileListEntry*	entry2 = (const FileListEntry*)arg2;

	return lstrcmp( (*(entry1->lines))[entry1->index].c_str(), (*(entry2->lines))[entry2->index].c_str() );
}

void CArea::sortFileList() const {
	LoggerPtr	logger;
	logger << LOG_ERROR;	// we log only errors here

	VTString				lines;
	typedef	vector<size_t>	Numbers;
	Numbers					indices;
	Numbers					lengths;

	// prepare file name of filelist
	tstring	fileListName( workingPath );
	fileListName += CCommonData::getInstance()->getFileListName();

	// open file for reading
	ifstream	in( fileListName.c_str() );

	// read file line by line
	const	BUFFER_SIZE = 1024;
	TCHAR	buffer[BUFFER_SIZE];
	while (!in.fail() && !in.eof()) {
		if (in.getline( buffer, BUFFER_SIZE )) {
			switch (buffer[0]) {
				case '\0':
					// ignore empty line
					break;
				case ' ':
				case '>':
				case '\t':
					// suppose that current line is a part of long description
					if (!lengths.empty()) {
						lines.push_back( buffer );
						++(lengths[lengths.size()-1]);
					}
					break;
				default:
					// suppose that current line contains filename and description
					indices.push_back( lines.size() );
					lines.push_back( buffer );
					lengths.push_back( 1 );
					break;
			}
		}
	}
	if (in.fail() && !in.eof()) {
		logger << TEXT("Failed to read all lines of filelist file,\n")
			   << TEXT("  file name = \"") << fileListName << TEXT("\"\n");
		return;
	}
	in.close();

	if (indices.empty()) {
		// there are no files in file list
		return;
	}

	// prepare array of FileListEntry elements to apply quick sort to it
	size_t			fileCount = indices.size();
	FileListEntry*	entries = new FileListEntry[fileCount];
	for (size_t	i = 0; i < fileCount; i++) {
		entries[i].lines  = &lines;
		entries[i].index  = indices[i];
		entries[i].length = lengths[i];
	}

	// sort array
	qsort( entries, fileCount, sizeof(FileListEntry), compareFileListEntries );

	// prepare temporary file name
	tstring	tempPath     = workingPath.substr( 0, workingPath.length() - 1 );	// we do not need last path separator
	tstring	tempFileName = getTempFileName( tempPath, CCommonData::getInstance()->getFileListName() );
	if (tempFileName.empty()) {
		logger << TEXT("Failed to get temporary file name,\n")
			   << TEXT("  path = \"") << tempPath << TEXT("\"");
		delete [] entries;
		return;
	}

	// open temp file for writing
	ofstream	out( tempFileName.c_str() );

	// save sorted lines
	for (size_t i = 0; i < fileCount && !out.fail(); i++) {
		size_t	lineCount = entries[i].length;
		size_t	pos       = entries[i].index;
		for (size_t j = 0; j < lineCount && !out.fail(); j++, pos++) {
			out << lines[pos].c_str() << endl;
		}
	}
	delete [] entries;
	if (out.fail()) {
		logger << TEXT("Failed to save sorted lines into temporary file,\n")
			   << TEXT("  file = \"") << tempFileName << TEXT("\"\n");
		return;
	}
	out.close();

	// remove old file list
	if (_tunlink( fileListName.c_str() ) == 0 || errno == ENOENT) {
		if (_trename( tempFileName.c_str(), fileListName.c_str() ) == 0) {
			if (!::SetFileAttributes( fileListName.c_str(), FILE_ATTRIBUTE_NORMAL )) {
				// we failed to set normal attribute
				logger << TEXT("failed to set normal attribute for the file:\n")
						<< TEXT("  file name = \"") << fileListName << TEXT("\"\n");
			}
		} else {
			// we failed to rename temp name to the real name
			logger << TEXT("failed to rename temporary file into real filelist file:\n")
					<< TEXT("  temp file name = \"") << tempFileName << TEXT("\"\n")
                    << TEXT("  real file name = \"") << fileListName << TEXT("\"\n");
		}
	} else {
		// we failed to remove old file
		logger << TEXT("failed to remove old filelist file:\n")
				<< TEXT("  file name = \"") << fileListName << TEXT("\"\n");
	}
}


void CArea::renameFile(const tstring& fullFileName, RenameFilePlan renamePlan) {
	tstring::size_type	pos;
	// extract file name without path
	tstring	fileName;
	pos = fullFileName.find_last_of( PATH_SEPARATOR );
	if (pos != tstring::npos) {
		fileName = fullFileName.substr( pos + 1 );
	} else {
		fileName = fullFileName;
	}
	// extract extension from file name
	tstring	ext;
	bool	hasExt = false;
	pos = fileName.find_last_of( '.' );
	if (pos != tstring::npos) {
		ext    = fileName.substr( pos + 1 );
		hasExt = true;
	}

	TCHAR	newFullFileName[MAX_PATH];
	lstrcpy( newFullFileName, fullFileName.c_str() );
	// prepare pointer to a part of file name that will be changed on each loop
	LPTSTR	mutablePart = newFullFileName + fullFileName.length();
	if (!hasExt || (renamePlan == RFP_ADD_EXT_SUFFIX)) {
		// if we have no extension we will use <name>.<number> format
		// if we have "extension suffix" plan we will use <name>.<ext>.<number> format
		*mutablePart = TEXT('.');
		++mutablePart;
	} else {
		// format is either <name>.<number>.<ext> or <name>.<newExt>, find position after dot
		mutablePart -= ext.length();
	}

	TCHAR	lastExt[MAX_PATH] = TEXT("");
	int		lastExtPos = -1;
	if (renamePlan == RFP_INCREASE_EXT) {
		// prepare extension that will be increased in each loop
		lstrcpy( lastExt, ext.c_str() );
		lastExtPos = lstrlen( lastExt )-1;
	}

	int	counter = 0;
	for (;;) {
		if (!hasExt || (renamePlan == RFP_ADD_EXT_SUFFIX)) {
			_itot( counter, mutablePart, 10 );
		} else {
			switch (renamePlan) {
			case RFP_INCREASE_EXT:
				{
					int	i = lastExtPos;
					for (;;) {
						if (AdrToUpper(lastExt[i]) < TEXT('A')) {
							lastExt[i] = 'A';
						} else {
							if (lastExt[i] < 255) {
								++lastExt[i];
							}
						}
						if (AdrToUpper(lastExt[i]) > TEXT('Z')) {
							// no more letters for the current position
							if (i == 0) {
								// no more positions
								// change rename plan to extension suffix
								lstrcpy( mutablePart, ext.c_str() );
								mutablePart += ext.length();
								*mutablePart = TEXT('.');
								++mutablePart;
								counter = 0;
								renamePlan = RFP_ADD_EXT_SUFFIX;
								_itot( counter, mutablePart, 10 );
								break;
							} else {
								// check original case
								if (AdrToUpper(ext.c_str()[i]) == ext.c_str()[i]) {
									lastExt[i] = TEXT('A');
								} else {
									lastExt[i] = TEXT('a');
								}
								// loop again to find extension
								--i;
							}
						} else {
							// next extension prepared
							lstrcpy( mutablePart, lastExt );
							break;
						}
					}
				}
				break;
			case RFP_ADD_EXT_PREFIX:
				_stprintf( mutablePart, TEXT("%d.%s"), counter, ext.c_str() );
				break;
			}
		}
		if (_trename( fullFileName.c_str(), newFullFileName ) == 0) {
			// good job!
			tstring	newFileName = newFullFileName + (fullFileName.length() - fileName.length());

			LoggerPtr	logger;
			logger << LOG_SILENT << TEXT("old file with name \"") << fileName
				   << TEXT("\" was renamed to \"") << newFileName << TEXT("\"\n");

			// update filelist

			tstring	descFileName = workingPath + CCommonData::getInstance()->getFileListName();
			TCHAR	tempFileName[MAX_PATH];
			lstrcpy( tempFileName, workingPath.c_str() );
			lstrcat( tempFileName, TEXT("adrenalin.tmp") );

			// enclose names in double quotes if names contain spaces
			if (fileName.find( TEXT(' ') ) != tstring::npos) {
				fileName = tstring(TEXT("\"")) + fileName + TEXT("\"");
			}
			if (newFileName.find( TEXT(' ') ) != tstring::npos) {
				newFileName = tstring(TEXT("\"")) + newFileName + TEXT("\"");
			}

			if (GetTmpFileName( tempFileName )) {
				std::ifstream	in( descFileName.c_str() );
				if (!in.fail()) {
					std::ofstream	out( tempFileName );

					bool	nameFound = false;
					char	buffer[1024];
					char	suspected[1024];
					while (!in.eof() && !out.fail()) {
						in.getline( buffer, 1024 );

						if (nameFound) {
							// simply copy line
							if (*buffer != '\0') {
								out << buffer << std::endl;
							}
						} else {
							lstrcpyn( suspected, buffer, fileName.length() + 1 );
							suspected[fileName.length()] = '\0';
							if (lstrcmpi( suspected, fileName.c_str() ) == 0 && 
								(buffer[fileName.length()] == '\0' || buffer[fileName.length()] == ' '))
							{
								out << newFileName.c_str() << (buffer + fileName.length()) << std::endl;
								nameFound = true;
							} else {
								out << buffer << std::endl;
							}
						}
					}

					if (out.fail()) {
						out.close();
						in.close();
						_tunlink( tempFileName );
					} else {
						out.close();
						in.close();
						_tunlink( descFileName.c_str() );
						_trename( tempFileName, descFileName.c_str() );
					}
				} else {
					_tunlink( tempFileName );
				}
			}

			break;
		} else {
			int	eNo = errno;
			if ((eNo != EACCES) && (eNo != EEXIST)) {
				// error occured
				LoggerPtr	logger;
				logger << LOG_ERROR << TEXT("Error occured while renaming \"") << fileName
					   << TEXT("\" to \"") << newFullFileName + (fullFileName.length() - fileName.length()) << TEXT("\"\n");
				break;
			}
			++counter;
		}
	}
}

