#pragma once

#include "exceptions.h"
#include "singletons.h"
#include "Areas.h"
#include "Common.h"
#include "Logger.h"
#include "LinksManager.h"

#include <MiniSAX.h>

using miniSAX::CCharSetConvertor;

class CAreasManager : public CSingleton
{
public:
	virtual ~CAreasManager();

	virtual void addArea(PArea newArea) { areas.push_back( newArea ); }
	virtual void addGroup(PAreaGroup newGroup) { groups.push_back( newGroup ); }
 
	PAreaGroup getGroup(const tstring& groupName);
	PAreaGroup getGroup(int index);

	int getGroupsCount() const { return groups.size(); }

	PArea getArea(const tstring& areaName);
	PArea getArea(int index) { return areas[index]; }

	int getAreasCount() const { return areas.size(); }

	PArea autoCreate(const tstring& areaName, PLink creator)
		throw(CLogicException);

	static CAreasManager* getInstance() throw(CBadConfigException) {
		static CAreasManager	areasManager;

		if (!areasManager.loaded) {
			areasManager.load();
		}

		return &areasManager;
	}

	void affectAreas() { areasChanged = true; }


	// singleton implementations


	virtual void destroy();

	virtual CSingletonVector getDependencies() {
		CSingletonVector	dependencies;
		dependencies.push_back( CCommonData::getInstance() );
		dependencies.push_back( CLogger::getInstance() );
		dependencies.push_back( CLinksManager::getInstance() );

		return dependencies;
	}

	// group iterator

	class CGroupIterator {
	public:
		CGroupIterator();
		CGroupIterator(CAreasManager* aManager, int anIndex);
		CGroupIterator(const CGroupIterator& instance);
		virtual ~CGroupIterator() {}

		CGroupIterator& operator =(const CGroupIterator& instance);

		bool operator ==(const CGroupIterator& instance);
		bool operator !=(const CGroupIterator& instance);

		CGroupIterator& operator ++();
		PAreaGroup operator *();

	protected:
		CAreasManager*	manager;
		int				index;
	};

	CGroupIterator	groupBegin();
	CGroupIterator	groupEnd();

	// area iterator

	class CAreaIterator {
	public:
		CAreaIterator();
		CAreaIterator(CAreasManager* aManager, PAreaGroup aGroup, int anAreaIndex);
		CAreaIterator(const CAreaIterator& instance);
		virtual ~CAreaIterator() {}

		CAreaIterator& operator =(const CAreaIterator& instance);

		bool operator ==(const CAreaIterator& instance);
		bool operator !=(const CAreaIterator& instance);

		CAreaIterator& operator ++();
		PArea operator *();

	protected:
		CAreasManager*	manager;
		PAreaGroup		group;
		int				areaIndex;
	};

	CAreaIterator areaBegin();
	CAreaIterator areaEnd();

	CAreaIterator areaBegin(PAreaGroup group);
	CAreaIterator areaEnd(PAreaGroup group);

	/**
	 * Remembers area for future processing (used to sort changed filelists of areas on exit).
	 * @return	true on success false on duplicate (area was already remembered);
	 *			areas are being compared as pointers
	 */
	bool rememberArea(const CArea* area);

protected:
	CAreasManager();
	// virtual bool parseAreaAccess(LPCTSTR s, PArea area, tstring& group);

	vector<PAreaGroup>	groups;

	typedef	vector<PArea>	Areas;

	Areas		areas;

	bool				areasChanged;

	tstring	m_documentEncoding;

	bool	loaded;
	void load() throw(CBadConfigException);
	void saveAreas();


	vector<const CArea*>	m_areasToRemember;
	void sortFileLists();
};
