#ifndef __KALUGA__MSG__HEADER__
#define __KALUGA__MSG__HEADER__

#include "FTNStruc.hpp"
#include "Common.h"
#include "Array.h"
#include "msgid.hpp"

// uncomment definition below to substitute Fidonet.org domain
// with Fidonet domain
#define SUBSTITUTE_DOMAIN

enum { SAVE_READ, SAVE_MIDDLE_OF_KLUDGE, SAVE_WRITE_LINE, SAVE_SKIP_LINE };

class CMsg {
private:
	uint16      m_wAttrs;
	uint16      m_wFlags;
	SYSTEMTIME  m_stTime;

	bool        m_fFMPTPresent;
	addr        m_wFMPT;

	bool        m_fTOPTPresent;
	addr        m_wTOPT;

	bool        m_fINTLPresent;
	CAddress	m_addrINTLFrom;
	CAddress	m_addrINTLTo;

	bool        m_fDOMAINPresent;
	CAddress	m_addrDomainKludgeFrom;
	CAddress	m_addrDomainKludgeTo;

	bool        m_fZoneGate;
	CAddress	m_addrRealTo;

	bool        m_fMsgIdPresent;
	bool        m_fMsgIdDecoded;
	CAddress	m_addrMsgIdFrom;

	TCHAR       m_szMsgIdLine[MAX_KLUDGE_SIZE];
	TCHAR       m_szArea[MAX_AREA_SIZE];
	HANDLE      m_hFile;
	bool        m_fMsgInBuf; // message in buffer or not
	DWORD       m_dwBodySize; // message body size
	CMsgId      m_MsgIdHandler;

	bool		m_fFlagsPresent;
	vector<tstring>	m_vFlags;

	TCHAR       m_szMsgFileName[MAX_PATH];
	TCHAR       m_szMsgFileDir[MAX_PATH];
	// address-treating functions
	void        DecodeAddresses();
	bool        ParsePT(LPCTSTR s, addr &PT);
	bool        ParseINTL(LPCTSTR s);
	bool        ParseFlags(LPCTSTR s);
	bool        ParseFTNAddress(LPCTSTR s, SFTNAddress &Address);
	bool        ParseMsgId(LPCTSTR s);
	bool        ParseDOMAIN(LPCTSTR s);
	// flags-treating fuctions
	void        SetFlag(LPCTSTR s);
	void        ClearFlag(LPCTSTR s);
	bool        CheckFlag(LPCTSTR s);
	// Adress-treating functions
	bool        FromNet(SFTNAddress &Addr) const; // checks Zone:Net
	bool        ToNet(SFTNAddress &Addr)  const;
	bool        FromNode(SFTNAddress &Addr)  const; // checks Zone:Net/Node
	bool        ToNode(SFTNAddress &Addr)  const;
	bool        FromPoint(SFTNAddress &Addr)  const; // checks Zone:Net/Node.Point
	bool        ToPoint(SFTNAddress &Addr)  const;
	// Name-treating functions
	bool        FromName(LPCTSTR szName) { return (lstrcmpi(m_Header.m_szFromUserName,
						       szName) == 0); }
	bool        ToName(LPCTSTR szName) { return (lstrcmpi(m_Header.m_szToUserName,
						     szName) == 0); }
	// Whole message treating functions
	/*
	bool        SaveMsg(FILE *tmp);
	void        Save();
	bool        Move(LPCTSTR szDir, long &FreeNumber);
	void        Bounce( uint16 wBounceAttrs, uint16 wBounceFlags,
			LPCTSTR szBounceTplFileName, SFTNAddress &BounceAddr );
	// user-defined fields and functions
	int         FromUs();
	int         ToUs();
	*/
	void	CloseFile() { CloseHandle(m_hFile); m_hFile = INVALID_HANDLE_VALUE; }
	CArray	m_arrMessage;

	/**
	 * Process message line
	 */
	void	ProcessLine(TCHAR kludgeTypeChar, LPTSTR lineData);

	SMsgHeader	m_Header;

	CAddress	m_fromAddress;
	CAddress	m_toAddress;
public:
	CMsg();
	~CMsg();

	void setSenderName(const std::string& name);
	void setReceiverName(const std::string& name);

	const char* getSenderName()		const { return m_Header.m_szFromUserName; }
	const char* getReceiverName()	const { return m_Header.m_szToUserName;   }
	const char* getSubject()		const { return m_Header.m_szSubject;      }

	void setFromAddress(const CAddress& address) { m_fromAddress = address; }
	const CAddress& getFromAddress() const { return m_fromAddress; }

	void setToAddress(const CAddress& address) { m_toAddress = address; }
	const CAddress& getToAddress() const { return m_toAddress; }

	TCHAR	m_szLastReplyFileName[MAX_PATH];

	long		m_lFreeMsgNumber;

	bool Read(LPCTSTR szMsgFName);
	void SetFreeMsgNumber(long lNumber) { m_lFreeMsgNumber = lNumber; }
	// these two are with domains (Zone:Net/Node.Point@Domain)
	bool  FromAddr(SFTNAddress &Addr);
	bool  ToAddr(SFTNAddress &Addr);
	bool  Delete();

	void Reply( uint16 wReplyAttrs, uint16 wReplyFlags, LPCTSTR szSubject,
		const vector<tstring> &vBodyLines );

	// reply with attributes and flags that are stored in CommonData
	void Reply(LPCTSTR szSubject, const vector<tstring> &vBodyLines) {
		CommonDataPtr	commonData = CCommonData::getInstance();
		Reply( commonData->getReplyMessageAttributes(), commonData->getReplyMessageFlags(),
			   szSubject, vBodyLines );
	}

	bool SaveAsReceived();
};

#endif