#pragma once
#include <MiniSAX.h>


/**
 * Suspended file
 */
class SuspendedFile {
public:
	SuspendedFile() : m_noCopy(false), m_deleteAfterHatch(false) {}
	~SuspendedFile() {}

	void setFileName(const tstring& name) { m_fileName = name; }
	const tstring& getFileName() const { return m_fileName; } 

	void setAreaName(const tstring& name) { m_areaName = name; }
	const tstring& getAreaName() const { return m_areaName; } 

	void setDescription(const tstring& desc) { m_desc = desc; }
	const tstring& getDescription() const { return m_desc; } 

	void setNoCopy(bool flag) { m_noCopy = flag; }
	bool isNoCopy() const { return m_noCopy; }

	void setReplaces(const tstring& replaces) { m_replaces = replaces; }
	const tstring& getReplaces() const { return m_replaces; } 

	void setReleaseDateTime(const SYSTEMTIME& dateTime) { m_releaseDateTime = dateTime; }
	const SYSTEMTIME& getReleaseDateTime() const { return m_releaseDateTime; }

	void setError(const tstring& error) { m_error = error; }
	const tstring& getError() const { return m_error; }
	bool hasError() const { return (!m_error.empty()); }

	void setDeleteAfterHatch(bool flag) { m_deleteAfterHatch = flag; }
	bool doDeleteAfterHatch() const { return m_deleteAfterHatch; }

protected:
	tstring		m_fileName;
	tstring		m_areaName;
	tstring		m_desc;
	bool		m_noCopy;
	tstring		m_replaces;
	SYSTEMTIME	m_releaseDateTime;
	tstring		m_error;
	bool		m_deleteAfterHatch;
};
typedef SuspendedFile*				SuspendedFilePtr;
typedef vector<SuspendedFilePtr>	SuspendedFiles;


class SuspendedHandler : public miniSAX::CContentHandler {
public:
	// second argument is the pointer to place where files will be stored
	SuspendedHandler(miniSAX::CMiniSAXParser* parser, SuspendedFiles* suspendedFiles);
    virtual ~SuspendedHandler();
	virtual void startElement(const char* name, miniSAX::CAttributeList& attributes)
        throw (miniSAX::CMiniSAXException);
    virtual void endElement(const char* name)
        throw (miniSAX::CMiniSAXException);
    virtual void handleCharacters(const char* buffer, size_t len)
        throw (miniSAX::CMiniSAXException);

	// all XML errors are handled inside

	static bool doesFileExist(const tstring& fileName);

	static bool readSuspendedFiles(const tstring& fileName, SuspendedFiles* suspendedFiles, tstring& documentEncoding);
	static void writeSuspendedFiles(const tstring& fileName, SuspendedFiles* suspendedFiles, const tstring& documentEncoding);
protected:
    miniSAX::CMiniSAXParser* m_parser;
    enum HandleState {
        ROOT_STATE,
        SUSPENDEDFILES_STATE,
        SUSPENDEDFILES_FILE_STATE,
        FILE_NAME_STATE,
        FILE_AREA_STATE,
        FILE_DESCRIPTION_STATE,
		FILE_NOCOPY_STATE,
        FILE_REPLACES_STATE,
        FILE_DATETIME_STATE,
		FILE_ERROR_STATE,
		FILE_DELETEAFTERHATCH_STATE
    };
    HandleState m_state;

    struct EntryDescriptor {
        HandleState m_state;
        const char* m_name;
    };
    typedef std::vector<EntryDescriptor>    EntriesHolder;
    EntriesHolder   m_entriesHolder;
    // ATTENTION: we compare pointers, not strings, so please submit only constants
    // this method throws exception if there is already an entry in m_entriesholder with the specified name
    // at the specified state
    void checkDup(LPCTSTR name, bool remember = true) throw(miniSAX::CMiniSAXException);
    // rememberes entry at the current state
    void rememberEntry(LPCTSTR name);
    // removes entry at the current state
    void removeEntry(LPCTSTR name);
    // removes all entries for current state
    void removeEntries();
    EntriesHolder::iterator getEntry(LPCTSTR name);
    bool hasEntry(LPCTSTR name);
    bool hasOnlySpaces(const char* s, size_t len);

	SuspendedFiles*		m_suspendedFiles;
	SuspendedFilePtr	m_currentFile;

	void parseUnsignedInt(const char* s, int& value) throw(miniSAX::CMiniSAXException);
};
