// HelpAction.cpp: implementation of the CHelpAction class.
//
//////////////////////////////////////////////////////////////////////
#include "pch.h"

#include "../Common.h"
#include "../exceptions.h"
#include "../Adrenalin.h"
#include "HelpAction.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHelpAction::CHelpAction(int aHelpType)
{
	helpType = aHelpType;
}


CHelpAction::~CHelpAction()
{

}


void CHelpAction::invoke() {
	switch (helpType) {
	case BRIEF_HELP:
		printBriefHelp();
		break;
	case DETAILED_HELP:
		printDetailedHelp();
		break;
	default:
		throw tstring(TEXT("Unknown help requested"));
	}
}


void CHelpAction::printBriefHelp() {
	std::cout << CAdrenalin::getName().c_str() << TEXT(" FileEchoProcessor for Windows\n");
	std::cout << TEXT("Copyright (c) Andrey Kuprishov 2:5023/27@Fidonet\n");
	std::cout << TEXT("Usage: Adrenalin.exe [options]\n");
	std::cout << TEXT("where options are:\n");
	std::cout << TEXT("  HELP        prints detailed help on options\n");
	std::cout << TEXT("  FIX         scans netmail directory for filefix requests\n");
	std::cout << TEXT("  TOSS        scans inbound directory for TIC-files and tosses them\n");
	std::cout << TEXT("  TOSSBAD     scans badtics directory for TIC-files and tosses them\n");
	std::cout << TEXT("  PURGE       [<area mask>... -<area mask>...]\n");
	std::cout << TEXT("  TIMEPURGE   <hours> [<address mask>...]\n");
	std::cout << TEXT("  BASEPURGE   <days> [areamask]+\n");
	std::cout << TEXT("  HATCH       <file> <area> <desc> [-r <Replaces>] [-L]\n");
	std::cout << TEXT("              [-release <date> [<time>] [-delete]]\n");
	std::cout << TEXT("  RELINK      <addr> [<robot name>]\n");
	std::cout << TEXT("  HAND        <addr> <command>+\n");
	std::cout << TEXT("  CHANGEGROUP (+|-)<group> <links mask>\n");
}


void CHelpAction::printDetailedHelp() {
	std::cout << CAdrenalin::getName().c_str() << TEXT(" FileEchoProcessor for Windows\n");
	std::cout << TEXT("Copyright (c) Andrey Kuprishov 2:5023/27@Fidonet\n");
	std::cout << TEXT("Usage: Adrenalin.exe [options]\n");
	std::cout << TEXT("where options are:\n");
	std::cout << TEXT("  HELP      prints this help information\n");
	std::cout << TEXT("  FIX       scans netmail directory for filefix requests\n");
	std::cout << TEXT("  TOSS      scans inbound directory for TIC-files and tosses them\n");
	std::cout << TEXT("  TOSSBAD   scans badtics directory for TIC-files and tosses them;\n");
	std::cout << TEXT("            files for TICs supposed to be in inbound directory\n");
	std::cout << TEXT("  PURGE     removes all sent files from passthrough or specified areas,\n");
	std::cout << TEXT("            e.g. PURGE <area mask>... -<area mask>...\n");
	std::cout << TEXT("  TIMEPURGE <hours> [<address mask>...]\n");
	std::cout << TEXT("            removes obsolete files and TICs (older than <hours> hours) from BSO\n");
	std::cout << TEXT("  BASEPURGE <days> [areamask]+\n");
	std::cout << TEXT("            removes obsolete files (older than <days> days) from fileechos\n");
	std::cout << TEXT("            using information from 'files.db' file\n");
	std::cout << TEXT("  HATCH <file> <area> <desc> [-r <Replaces>] [-L]\n");
	std::cout << TEXT("        [-release <date> <time> [-delete]]\n");
	std::cout << TEXT("            hatches file into fileecho\n");
	std::cout << TEXT("              <file>     name of file to hatch\n");
	std::cout << TEXT("              <area>     fileecho area to be hatched into\n");
	std::cout << TEXT("              <desc>     description of file\n");
	std::cout << TEXT("              <Replaces> optional Replaces file name\n");
	std::cout << TEXT("              -L         means that file will be left without copying it into\n");
	std::cout << TEXT("                         the fileecho directory\n");
	std::cout << TEXT("              -release   suspended hatch, file will be hatched automatically\n");
	std::cout << TEXT("                         after release date (on the first toss),\n");
	std::cout << TEXT("                         date can be specified either as DD.MM or as DD.MM.YYYY\n");
	std::cout << TEXT("                         time can be specified either as HH:MM or as HH:MM:SS,\n");
	std::cout << TEXT("                         if time is not specified 00:00:00 assumed\n");
	std::cout << TEXT("              -delete    file will be deleted after hatch (for release only)\n");
	std::cout << TEXT("  RELINK <addr> [<robot name>]\n");
	std::cout << TEXT("            sends relink request to the specified link using robot name as\n");
	std::cout << TEXT("            a receiver name (in a field TO)\n");
	std::cout << TEXT("            default robot name is AllFix\n");
	std::cout << TEXT("  HAND <addr> <command>+\n");
	std::cout << TEXT("            simulates filefix request from a link at <addr> address.\n");
	std::cout << TEXT("  CHANGEGROUP (+|-)<group> <links mask>\n");
	std::cout << TEXT("              changes group for links (addes or removes), example:\n");
	std::cout << TEXT("              changegroup +BOOK */27.*\n");
	std::cout << TEXT("              will add group BOOK to all my (and every /27) points\n");
}
